'use strict';

exports.__esModule = true;

var _merge = require('ramda/src/merge');

var _merge2 = _interopRequireDefault(_merge);

var _identity = require('ramda/src/identity');

var _identity2 = _interopRequireDefault(_identity);

var _T = require('ramda/src/T');

var _T2 = _interopRequireDefault(_T);

var _ = require('ramda/src/__');

var _2 = _interopRequireDefault(_);

var _applyTo = require('ramda/src/applyTo');

var _applyTo2 = _interopRequireDefault(_applyTo);

var _is = require('ramda/src/is');

var _is2 = _interopRequireDefault(_is);

var _cond = require('ramda/src/cond');

var _cond2 = _interopRequireDefault(_cond);

var _keys = require('ramda/src/keys');

var _keys2 = _interopRequireDefault(_keys);

var _curry = require('ramda/src/curry');

var _curry2 = _interopRequireDefault(_curry);

var _util = require('./util');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * A port of [Ramda's evolve()](http://ramdajs.com/docs/#evolve), but the
 * transforms are _always_ applied regardless if the key/value pair exists in
 * the input object or not.
 * 
 *
 * @func
 * @sig {k: (a -> b)} -> {k: v} -> {k: v}
 * @param {Object} transforms An object whose values may be transform functions that
 * need to be wrapped in try/catch
 * @param {Object} input The input object to pass through prop-level transformations
 * @returns {Object} The modified input object, with any prop-level transforms
 * applied to it
 */
var alwaysEvolve = (0, _curry2.default)(function (transforms, input) {
  var result = {};
  var spec = (0, _util.safeSpecTransforms)(transforms);
  var object = (0, _util.objectify)(input);
  (0, _keys2.default)(spec).forEach(function (key) {
    result[key] = (0, _cond2.default)([[(0, _is2.default)(Function), (0, _applyTo2.default)(object[key])], [_util.isPlainObj, alwaysEvolve(_2.default, object[key])], [_T2.default, _identity2.default]])(spec[key]);
  });
  return (0, _merge2.default)(input, result);
});

exports.default = alwaysEvolve;