'use strict';

exports.__esModule = true;

var _has = require('ramda/src/has');

var _has2 = _interopRequireDefault(_has);

var _T = require('ramda/src/T');

var _T2 = _interopRequireDefault(_T);

var _ = require('ramda/src/__');

var _2 = _interopRequireDefault(_);

var _applyTo = require('ramda/src/applyTo');

var _applyTo2 = _interopRequireDefault(_applyTo);

var _is = require('ramda/src/is');

var _is2 = _interopRequireDefault(_is);

var _cond = require('ramda/src/cond');

var _cond2 = _interopRequireDefault(_cond);

var _keys = require('ramda/src/keys');

var _keys2 = _interopRequireDefault(_keys);

var _curry = require('ramda/src/curry');

var _curry2 = _interopRequireDefault(_curry);

var _util = require('./util');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * A port of [Ramda's evolve()](http://ramdajs.com/docs/#evolve), but any
 * primitive value or array in the transform spec is wrapped in a function that
 * always returns that value. Any functions already on the transform spec are
 * applied at the prop-level (like evolve() always does) and any objects on the
 * transform spec call the evolver recursively (again, normal behavior for
 * evolve()).
 * In other words, the only difference between this and Ramda's
 * evolve() function is that primitives and array vales are wrapped in an
 * always() function before the spec is fed to evolve() rather than being
 * omitted from the final output.
 *
 * @func
 * @sig {k: (a -> b)} -> {k: v} -> {k: v}
 * @param {Object} transforms An object with transform functions or nested transform
 * functions or primitive vales to be wrapped with a function.
 * @param {Object} input The input object to pass through prop-level transformations
 * @returns {Object} The modified input object, with any prop-level transforms
 * applied to it
 */
var evolveSpec = (0, _curry2.default)(function (transforms, input) {
  var result = {};
  var spec = (0, _util.safeSpecTransforms)(transforms);
  var object = (0, _util.objectify)(input);
  (0, _keys2.default)(object).forEach(function (key) {
    result[key] = (0, _cond2.default)([[(0, _is2.default)(Function), (0, _applyTo2.default)(object[key])], [_util.isPlainObj, evolveSpec(_2.default, object[key])], [_T2.default, function (val) {
      return (0, _has2.default)(key, spec) ? val : object[key];
    }]])(spec[key]);
  });
  return result;
});

exports.default = evolveSpec;