'use strict';

exports.__esModule = true;

var _curry = require('ramda/src/curry');

var _curry2 = _interopRequireDefault(_curry);

var _apply = require('ramda/src/apply');

var _apply2 = _interopRequireDefault(_apply);

var _values = require('ramda/src/values');

var _values2 = _interopRequireDefault(_values);

var _pluck = require('ramda/src/pluck');

var _pluck2 = _interopRequireDefault(_pluck);

var _max = require('ramda/src/max');

var _max2 = _interopRequireDefault(_max);

var _reduce = require('ramda/src/reduce');

var _reduce2 = _interopRequireDefault(_reduce);

var _curryN = require('ramda/src/curryN');

var _curryN2 = _interopRequireDefault(_curryN);

var _ifElse = require('ramda/src/ifElse');

var _ifElse2 = _interopRequireDefault(_ifElse);

var _map = require('ramda/src/map');

var _map2 = _interopRequireDefault(_map);

var _util = require('./util');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var makeSpec = function makeSpec(transforms) {
  var spec = (0, _map2.default)((0, _ifElse2.default)(_util.isPlainObj, makeSpec, _util.alwaysFunction), (0, _util.safeSpecTransforms)(transforms));
  return (0, _curryN2.default)((0, _reduce2.default)(_max2.default, 0, (0, _pluck2.default)('length', (0, _values2.default)(spec))), function () {
    for (var _len = arguments.length, allArgs = Array(_len), _key = 0; _key < _len; _key++) {
      allArgs[_key] = arguments[_key];
    }

    return (0, _map2.default)(function (fn) {
      return (0, _apply2.default)(fn, allArgs);
    }, spec);
  });
};

/**
 * A port over of [Ramda's applySpec()](http://ramdajs.com/docs/#applySpec),
 * but with a few changes you may or may not see as necessary.
 * (1) Properly curried so that you can optionally provide args as (spec, input) rather than only as (spec)(input)
 * (2) It wraps each function in a tryCatch that logs the key name of the transform function that fails.
 *
 * Using applySpec() in the wild and at-scale proved to be one of the hardest Ramda functions to debug.
 * Limiting its functionality and adding console logging doesn't sound like a necessary thing,
 * but when working with Ramda on a full team it may help prevent devs from getting so frustrated
 * that they vote to abandon the library while they're still learning the functional programming paradigm.
 *
 * @func
 * @sig {k: v} -> {k: v} -> {k: v}
 * @param {Object} spec An object whose values are functions which
 * expect the entire object to be fed in as input.
 * This is identical to the input you'd pass into applySpec()
 * @param {Object} input An object to be mapped over (transformed)
 * @returns {Object} The input object transformed to contain only the props named in the spec
 * (with the corresponding transforms applied at that key)
 */
exports.default = (0, _curry2.default)(function (transforms, input) {
  return makeSpec(transforms)(input);
});