'use strict';

exports.__esModule = true;
exports.shapeline = exports.shapeStrictly = exports.shapeLoosely = exports.applyWholeObjectTransforms = undefined;

var _map = require('ramda/src/map');

var _map2 = _interopRequireDefault(_map);

var _reduce = require('ramda/src/reduce');

var _reduce2 = _interopRequireDefault(_reduce);

var _omit = require('ramda/src/omit');

var _omit2 = _interopRequireDefault(_omit);

var _T = require('ramda/src/T');

var _T2 = _interopRequireDefault(_T);

var _test = require('ramda/src/test');

var _test2 = _interopRequireDefault(_test);

var _pathSatisfies = require('ramda/src/pathSatisfies');

var _pathSatisfies2 = _interopRequireDefault(_pathSatisfies);

var _both = require('ramda/src/both');

var _both2 = _interopRequireDefault(_both);

var _cond = require('ramda/src/cond');

var _cond2 = _interopRequireDefault(_cond);

var _always = require('ramda/src/always');

var _always2 = _interopRequireDefault(_always);

var _isEmpty = require('ramda/src/isEmpty');

var _isEmpty2 = _interopRequireDefault(_isEmpty);

var _when = require('ramda/src/when');

var _when2 = _interopRequireDefault(_when);

var _identity = require('ramda/src/identity');

var _identity2 = _interopRequireDefault(_identity);

var _merge = require('ramda/src/merge');

var _merge2 = _interopRequireDefault(_merge);

var _converge = require('ramda/src/converge');

var _converge2 = _interopRequireDefault(_converge);

var _pipe = require('ramda/src/pipe');

var _pipe2 = _interopRequireDefault(_pipe);

var _is = require('ramda/src/is');

var _is2 = _interopRequireDefault(_is);

var _filter = require('ramda/src/filter');

var _filter2 = _interopRequireDefault(_filter);

var _difference = require('ramda/src/difference');

var _difference2 = _interopRequireDefault(_difference);

var _applyTo = require('ramda/src/applyTo');

var _applyTo2 = _interopRequireDefault(_applyTo);

var _keys = require('ramda/src/keys');

var _keys2 = _interopRequireDefault(_keys);

var _pick = require('ramda/src/pick');

var _pick2 = _interopRequireDefault(_pick);

var _compose = require('ramda/src/compose');

var _compose2 = _interopRequireDefault(_compose);

var _curry = require('ramda/src/curry');

var _curry2 = _interopRequireDefault(_curry);

var _prune = require('./prune');

var _util = require('./util');

var _alwaysEvolve = require('./alwaysEvolve');

var _alwaysEvolve2 = _interopRequireDefault(_alwaysEvolve);

var _evolveSpec = require('./evolveSpec');

var _evolveSpec2 = _interopRequireDefault(_evolveSpec);

var _mapSpec = require('./mapSpec');

var _mapSpec2 = _interopRequireDefault(_mapSpec);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Applies transform functions in a given spec to a given object, BUT only the
 * trasnforms that DON'T match key names on the input object are applied.
 * 
 * @func
 * @sig {k: v} -> {k: v} -> {k: v}
 * @param {Object} spec An object which contains transform functions (only those which do not correspond to keys on the input object will be applied to the input object)
 * @param {Object} input An object which will be passed through the re-shaping transform functions defined by the spec
 * @returns {Object} The input object with the spec transforms applied to it
 * (only those that do not correspond to key names on the input object)
 */
var applyWholeObjectTransforms = exports.applyWholeObjectTransforms = (0, _curry2.default)(function (spec, value) {
  return (0, _compose2.default)(_mapSpec2.default, (0, _applyTo2.default)(spec), _pick2.default, (0, _difference2.default)((0, _keys2.default)((0, _filter2.default)((0, _is2.default)(Function), spec))), _keys2.default)(value);
});

var baseShaper = (0, _curry2.default)(function (evolver, spec, value) {
  return (0, _pipe2.default)((0, _util.applyNonTransformProps)(spec), evolver((0, _util.onlySpecTransforms)(spec)), (0, _converge2.default)(_merge2.default, [_identity2.default, (0, _converge2.default)(_applyTo2.default, [(0, _when2.default)(_isEmpty2.default, (0, _always2.default)(value)), applyWholeObjectTransforms(spec)])]))(value);
});

/**
 * This function allows one to blend an object with a clone that is taken
 * through prop transformations. Unlike object mappers/transformers, not every
 * value in the spec need be a function. Non-function values are simply added to
 * the result. If a given value in the spec IS a function however, it will be
 * used to transform OR to create a prop. If the prop specified on the spec
 * exists on the value (object) being mapped, a prop-level transform will be
 * perform (just like [Ramda's evolve()](http://ramdajs.com/docs/#evolve)).
 * But if a prop in the spec doesn't exist in the object and if that prop is a
 * transformer function, then the transformer will be used to create that prop
 * on the resulting object BUT the entire object will be passed into the
 * transform function (just like [Ramda's applySpec()](http://ramdajs.com/docs/#applySpec), rather than just a single prop.
 *
 * @func
 * @sig {k: v} -> {k: v} -> {k: v}
 * @param {Object} spec An object whose values are either transform functions or pass-through values to be added to the return object
 * @param {Object} input An object which will be passed through the re-shaping transform functions defined by the spec
 * @returns {Object} The input object with the spec transforms applied to it
 */
var shapeLoosely = exports.shapeLoosely = baseShaper(_evolveSpec2.default);

/**
 * Applies a shaping spec to an input object, but will NOT pass through any props unless they are named in the spec.
 * In other words, you're providing it template for creating a brand new object from some raw input.
 *
 * @func
 * @sig {k: v} -> {k: v} -> {k: v}
 * @param {Object} spec An object whose values are either transform functions or pass-through values to be added to the return object
 * @param {Object} input An object which will be passed through the re-shaping transform functions defined by the spec
 * @returns {Object} The input object with the spec transforms applied to it
 */
var shapeStrictly = exports.shapeStrictly = (0, _curry2.default)(function (spec, input) {
  return (0, _compose2.default)((0, _pick2.default)((0, _keys2.default)(spec)), shapeLoosely(spec))(input);
});

var pruneOutput = (0, _curry2.default)(function (spec, input) {
  return (0, _when2.default)(_util.isPlainObj, (0, _cond2.default)([[(0, _is2.default)(Function), _identity2.default], [(0, _both2.default)(_util.isPlainObj, (0, _pathSatisfies2.default)((0, _test2.default)(/remove/i), ['shapeyMode'])), _prune.remover], [(0, _both2.default)(_util.isPlainObj, (0, _pathSatisfies2.default)((0, _test2.default)(/keep/i), ['shapeyMode'])), _prune.keeper], [(0, _both2.default)(_util.isPlainObj, (0, _pathSatisfies2.default)((0, _test2.default)(/strict/i), ['shapeyMode'])), function (sp) {
    return (0, _pipe2.default)((0, _prune.impliedRemove)(sp), (0, _util.applyNonTransformProps)(sp));
  }], [_util.isPlainObj, _util.applyNonTransformProps], [_T2.default, _always2.default]])(spec))(input);
});

var applyTransforms = (0, _curry2.default)(function (spec, input) {
  return (0, _cond2.default)([[(0, _is2.default)(Function), _identity2.default], [(0, _both2.default)(_util.isPlainObj, (0, _pathSatisfies2.default)((0, _test2.default)(/prop/i), ['shapeyTransforms'])), (0, _pipe2.default)(_util.onlySpecTransforms, _alwaysEvolve2.default)], [(0, _both2.default)(_util.isPlainObj, (0, _pathSatisfies2.default)((0, _test2.default)(/whole/i), ['shapeyTransforms'])), (0, _pipe2.default)(_util.onlySpecTransforms, _mapSpec2.default)], [_util.isPlainObj, (0, _pipe2.default)(_util.onlySpecTransforms, shapeLoosely)], [_T2.default, _always2.default]])(spec)(input);
});

/**
 * A single function that selects and applies one of the available re-shaping functions in the shapey library,
 * depending on what you've set the `shapeyMode` prop to in the `spec` you provide as the first argument to this function.
 *
 * Think of it like a case statement in a Redux reducer, however since you most likely _don't_ want to sacrifice
 * the meanining associated with the "type" property to internals of shapey, a prop called "shapeyMode" is used instead.
 *
 * If for some reason you have some prop on the input (the object you're transforming) already named "shapeyMode", um . . . don't.
 *
 * Available modes (case & space in-sensitive):
 *   "strict" - uses `shapeStrictly`, where _only_ the props named in your spec are included in the output
 *   "keep" - uses `keepAndShape`, where all the props you name in your spec are kept.
 *   "remove" - uses `removeAndShape`, where all the props you name in your spec are removed.
 *
 * In addition to controlling the mode for Shapey, you can control how the transforms are applied.
 * This is controlled via the reserved prop in your spec called "shapeyTransforms", and the available options for it are:
 *   "prop" - All transforms are applied at the prop-level, regardless if they exist on the input object
 *   "whole" - All transforms are given the _entire_ input object as input
 *   (regardless if a prop matching the name of the transform exists on the input object)
 *
 * @func
 * @sig {k: v} -> {k: v} -> {k: v}
 * @param {Object} spec An object whose values are either transform functions or pass-through values to be added to the return object
 * @param {Object} input An object which will be passed through the re-shaping transform functions defined by the spec
 * @returns {Object} The input object with the spec transforms applied to it
 */
var makeShaper = (0, _curry2.default)(function (spec, input) {
  return (0, _pipe2.default)(applyTransforms(spec), pruneOutput(spec), (0, _when2.default)(_util.isPlainObj, (0, _omit2.default)(['shapeyTransforms', 'shapeyMode'])))(input);
});

/**
 * Applies a list of functions (in sequence) to a single input, passing
 * the transformed output as the input value to the next function in the chain.
 * If a spec object is included in the transforms, the shaper that corresponds
 * to the "shapeyMode" prop is invoked (otherwise `shapeLoosely()` is used).
 * This creates a transform function for the pipeline.
 *
 * @func
 * @sig [a -> b, b -> c, c -> d, ...] -> * -> *
 * @param {Function[]} transforms A list of transform functions
 * @param {*} input The input value to pass through the enhancer pipeline
 * @returns {*} The output of the original input passed through the chain of transform functions
 */
var shapeline = exports.shapeline = (0, _curry2.default)(function (transforms, input) {
  return (0, _reduce2.default)(function (inputObj, fn) {
    return fn(inputObj);
  }, input, (0, _map2.default)(makeShaper, transforms || []));
});

exports.default = makeShaper;