'use strict';

exports.__esModule = true;
exports.safeSpecTransforms = exports.applyNonTransformProps = exports.removeMagicProps = exports.onlySpecTransforms = exports.makePruningSpec = exports.makeFunctionUnlessObject = exports.alwaysFunction = exports.objectify = exports.isPlainObj = undefined;

var _omit = require('ramda/src/omit');

var _omit2 = _interopRequireDefault(_omit);

var _tryCatch = require('ramda/src/tryCatch');

var _tryCatch2 = _interopRequireDefault(_tryCatch);

var _mapObjIndexed = require('ramda/src/mapObjIndexed');

var _mapObjIndexed2 = _interopRequireDefault(_mapObjIndexed);

var _identity = require('ramda/src/identity');

var _identity2 = _interopRequireDefault(_identity);

var _T = require('ramda/src/T');

var _T2 = _interopRequireDefault(_T);

var _equals = require('ramda/src/equals');

var _equals2 = _interopRequireDefault(_equals);

var _cond = require('ramda/src/cond');

var _cond2 = _interopRequireDefault(_cond);

var _prop = require('ramda/src/prop');

var _prop2 = _interopRequireDefault(_prop);

var _assoc = require('ramda/src/assoc');

var _assoc2 = _interopRequireDefault(_assoc);

var _converge = require('ramda/src/converge');

var _converge2 = _interopRequireDefault(_converge);

var _merge = require('ramda/src/merge');

var _merge2 = _interopRequireDefault(_merge);

var _curry = require('ramda/src/curry');

var _curry2 = _interopRequireDefault(_curry);

var _test = require('ramda/src/test');

var _test2 = _interopRequireDefault(_test);

var _reject = require('ramda/src/reject');

var _reject2 = _interopRequireDefault(_reject);

var _when = require('ramda/src/when');

var _when2 = _interopRequireDefault(_when);

var _filter = require('ramda/src/filter');

var _filter2 = _interopRequireDefault(_filter);

var _fromPairs = require('ramda/src/fromPairs');

var _fromPairs2 = _interopRequireDefault(_fromPairs);

var _toPairs = require('ramda/src/toPairs');

var _toPairs2 = _interopRequireDefault(_toPairs);

var _pipe = require('ramda/src/pipe');

var _pipe2 = _interopRequireDefault(_pipe);

var _either = require('ramda/src/either');

var _either2 = _interopRequireDefault(_either);

var _is = require('ramda/src/is');

var _is2 = _interopRequireDefault(_is);

var _always = require('ramda/src/always');

var _always2 = _interopRequireDefault(_always);

var _unless = require('ramda/src/unless');

var _unless2 = _interopRequireDefault(_unless);

var _pathEq = require('ramda/src/pathEq');

var _pathEq2 = _interopRequireDefault(_pathEq);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * This is what you really mean when you're checking to see if something is an Object.
 * Honestly who cares that an Array, Date and RegExp are technically objects too?
 * You were looking for the thing with the curly braces {} right?
 *
 * @func
 * @sig * -> Boolean
 * @param {*} val A value of any type
 * @returns {Boolean} Whether or not the value is an Object
 */
var isPlainObj = exports.isPlainObj = (0, _pathEq2.default)(['constructor', 'name'], 'Object');

/**
 * Turns any non-Object values into empty objects 
 *
 * @func
 * @sig * -> {k: v}
 * @param {*} val A value of any type
 * @returns {Object} If the original value was an Object, it is returned as-is,
 * otherwise an empty object is returned
 */
var objectify = exports.objectify = (0, _unless2.default)(isPlainObj, (0, _always2.default)({}));

/**
 * If the provided value is NOT a function, transform it into a function that
 * always returns that value 
 *
 * @func
 * @sig * -> (* -> *)
 * @param {*} val A value of any type
 * @returns {Function} If the original value was a function, that function is returned,
 * otherwise a function that always returns the original value is returned
 * function that always returns that value OR 
 */
var alwaysFunction = exports.alwaysFunction = (0, _unless2.default)((0, _is2.default)(Function), _always2.default);

/**
 * If the provided value is neither an Object nor a Function, transform it into
 * a function that always returns that provided value.
 *
 * @func
 * @sig * -> (* -> *)|{k: v}
 * @param {*} val A value of any type
 * @returns {Function|Object} A function that always returns the provided value
 * OR the original value (if it was already a Function or an Object)
 */
var makeFunctionUnlessObject = exports.makeFunctionUnlessObject = (0, _unless2.default)((0, _either2.default)((0, _is2.default)(Function), isPlainObj), _always2.default);

/**
 * Makes sure a given spec is acceptable for one of the pruning modes (ie, "remove" or "keep"). 
 * Specs in this mode are treated more strictly, meaning the prop must be given
 * a value of "true" or the key and value can be identical.
 * Functions are acceptable values in the spec of course.
 *
 * @func
 * @sig {k: v} -> {k: v}
 * @param {Object} spec A spec to be coerced into an acceptable pruning spec
 * @returns {Object} A spec that is acceptable to be used in pruning mode.
 */
var makePruningSpec = exports.makePruningSpec = (0, _pipe2.default)(objectify, _toPairs2.default, (0, _filter2.default)(function (_ref) {
  var key = _ref[0],
      val = _ref[1];
  return key === val || val === true || (0, _is2.default)(Function, val);
}), _fromPairs2.default);

/**
 * Filters a spec (object) to only the props that are transform functions. If
 * the input is not an object, this is just an identity function. 
 *
 * @func
 * @sig {k: v} -> {k: v}
 * @param {Object} spec An object whose values (may) be transform functions
 * @returns {Object} The input object with only the props that are functions retained
 */
var onlySpecTransforms = exports.onlySpecTransforms = (0, _when2.default)(isPlainObj, (0, _pipe2.default)(_toPairs2.default, (0, _filter2.default)(function (_ref2) {
  var key = _ref2[0],
      val = _ref2[1];
  return key === 'shapeyDebug' || (0, _is2.default)(Function, val);
}), _fromPairs2.default));

/**
 * Removes the reserved "shapey*" prefixed props from a spec.
 *
 * @func
 * @sig {k: v} -> {k: v}
 * @param {Object} spec A shapey spec to be pruned of magic props
 * @returns {Object} A shapey spec cleaned of magic props
 */
var removeMagicProps = exports.removeMagicProps = (0, _pipe2.default)(_toPairs2.default, (0, _reject2.default)(function (_ref3) {
  var key = _ref3[0];
  return (0, _test2.default)(/^shapey/i, key);
}), _fromPairs2.default);

/**
 * Takes an non-transform props (non-functions) in a given spec and merges them onto the input object.
 * 
 * @func
 * @sig {k: v} -> {k: v} -> {k: v}
 * @param {Object} spec An object whose values are either transform functions or pass-through values to be added to the return object
 * @param {Object} input An object which will be passed through the re-shaping transform functions defined by the spec
 * @returns {Object} The input object with only the pass-through props applied to it
 */
var applyNonTransformProps = exports.applyNonTransformProps = (0, _curry2.default)(function (spec, value) {
  return [spec, value].every(isPlainObj) ? (0, _pipe2.default)(removeMagicProps, (0, _reject2.default)((0, _is2.default)(Function)), (0, _merge2.default)(value))(spec) : value;
});

/**
 * Logs to the console any failed transform functions, along with the field name
 * and the value that was fed into the transform function. Plus, the exception
 * itself is also logged.
 * 
 * @func
 * @sig String -> * -> Error -> *
 * @param {String} fieldName The field name for the failed transform function
 * @param {Object} value The value that was fed into the transform function
 * @param {Object} exception The exception thrown by the failed transform
 * @returns {undefined}
 */
var defaultErrorHandler = (0, _curry2.default)(function (fieldName, value, exception) {
  return (
    // eslint-disable-next-line no-console
    console.error('\n Transform failed on field: "' + fieldName + '"\n value:', value, '\n', exception)
  );
});

/**
 * A wrapper around an (optional) error handler that the may be passed in as the
 * "shapeyDebug" value. It is curried, but will feed their handler a more
 * standard signature of: (err, field, value)
 *
 * @func
 * @sig ((Error, String, *) -> *) -> String -> * -> Error -> *
 * @param {Function} errHandler The custom error handler provided by the consumer
 * @param {String} fieldName The field name for the failed transform function
 * @param {Object} value The value that was fed into the transform function
 * @param {Object} exception The exception thrown by the failed transform
 * @returns {*} Could be anything, or nothing (it's up to the consumer)
 */
var wrapperForTheirErrorHandler = (0, _curry2.default)(function (errHandler, fieldName, value, exception) {
  return errHandler(exception, fieldName, value);
});

/**
 * Wraps every function on a given spec in a try/catch that catches any exception.
 *
 * What it _does_ with the exception is up to the consumer:
 *   - ignores it (if "shapeyDebug" isn't set)
 *   - logs it along with the corresponding field name,
 *     using `console.error` (if "shapeyDebug" is set to `true`)
 *   - uses a custom handler supplied by the consumer as the value for "shapeyDebug" 
 *
 * Note that a custom error handler will be passed the following params (in order):
 *   - The exception
 *   - The field name
 *   - The value that was fed into the transform function
 *
 * @func
 * @sig {k: (a -> b)} -> {k: (a -> b)}
 * @param {Object} spec An object whose values may be transform functions that
 * need to be wrapped in try/catch
 * @returns {Object} The same spec object but whose functions are
 * safely wrapped in in try/catch handlers
 */
var safeSpecTransforms = exports.safeSpecTransforms = (0, _pipe2.default)(objectify, (0, _converge2.default)((0, _assoc2.default)('shapeyDebug'), [(0, _pipe2.default)((0, _prop2.default)('shapeyDebug'), (0, _cond2.default)([[(0, _equals2.default)(true), (0, _always2.default)(defaultErrorHandler)], [(0, _is2.default)(Function), wrapperForTheirErrorHandler], [(0, _test2.default)(/^skip$/i), function () {
  return function (_, originalValue) {
    return (0, _always2.default)(originalValue);
  };
}], [_T2.default, function () {
  return function () {
    return (0, _always2.default)(undefined);
  };
}]])), _identity2.default]), (0, _mapObjIndexed2.default)(function (transform, fieldName, obj) {
  return (0, _when2.default)((0, _is2.default)(Function), (0, _curry2.default)(function (fn, val) {
    return (0, _tryCatch2.default)(fn, obj.shapeyDebug(fieldName, val))(val);
  }))(transform);
}), (0, _omit2.default)(['shapeyDebug']));