import _default9 from 'ramda/src/merge';
import _default8 from 'ramda/src/identity';
import _default7 from 'ramda/src/T';
import _default6 from 'ramda/src/__';
import _default5 from 'ramda/src/applyTo';
import _default4 from 'ramda/src/is';
import _default3 from 'ramda/src/cond';
import _default2 from 'ramda/src/keys';
import _default from 'ramda/src/curry';

import { safeSpecTransforms, isPlainObj, objectify } from './util';

/**
 * A port of [Ramda's evolve()](http://ramdajs.com/docs/#evolve), but the
 * transforms are _always_ applied regardless if the key/value pair exists in
 * the input object or not.
 * 
 *
 * @func
 * @sig {k: (a -> b)} -> {k: v} -> {k: v}
 * @param {Object} transforms An object whose values may be transform functions that
 * need to be wrapped in try/catch
 * @param {Object} input The input object to pass through prop-level transformations
 * @returns {Object} The modified input object, with any prop-level transforms
 * applied to it
 */
var alwaysEvolve = _default(function (transforms, input) {
  var result = {};
  var spec = safeSpecTransforms(transforms);
  var object = objectify(input);
  _default2(spec).forEach(function (key) {
    result[key] = _default3([[_default4(Function), _default5(object[key])], [isPlainObj, alwaysEvolve(_default6, object[key])], [_default7, _default8]])(spec[key]);
  });
  return _default9(input, result);
});

export default alwaysEvolve;