import _default8 from 'ramda/src/has';
import _default7 from 'ramda/src/T';
import _default6 from 'ramda/src/__';
import _default5 from 'ramda/src/applyTo';
import _default4 from 'ramda/src/is';
import _default3 from 'ramda/src/cond';
import _default2 from 'ramda/src/keys';
import _default from 'ramda/src/curry';

import { safeSpecTransforms, isPlainObj, objectify } from './util';

/**
 * A port of [Ramda's evolve()](http://ramdajs.com/docs/#evolve), but any
 * primitive value or array in the transform spec is wrapped in a function that
 * always returns that value. Any functions already on the transform spec are
 * applied at the prop-level (like evolve() always does) and any objects on the
 * transform spec call the evolver recursively (again, normal behavior for
 * evolve()).
 * In other words, the only difference between this and Ramda's
 * evolve() function is that primitives and array vales are wrapped in an
 * always() function before the spec is fed to evolve() rather than being
 * omitted from the final output.
 *
 * @func
 * @sig {k: (a -> b)} -> {k: v} -> {k: v}
 * @param {Object} transforms An object with transform functions or nested transform
 * functions or primitive vales to be wrapped with a function.
 * @param {Object} input The input object to pass through prop-level transformations
 * @returns {Object} The modified input object, with any prop-level transforms
 * applied to it
 */
var evolveSpec = _default(function (transforms, input) {
  var result = {};
  var spec = safeSpecTransforms(transforms);
  var object = objectify(input);
  _default2(object).forEach(function (key) {
    result[key] = _default3([[_default4(Function), _default5(object[key])], [isPlainObj, evolveSpec(_default6, object[key])], [_default7, function (val) {
      return _default8(key, spec) ? val : object[key];
    }]])(spec[key]);
  });
  return result;
});

export default evolveSpec;