import _default9 from 'ramda/src/curry';
import _default8 from 'ramda/src/apply';
import _default7 from 'ramda/src/values';
import _default6 from 'ramda/src/pluck';
import _default5 from 'ramda/src/max';
import _default4 from 'ramda/src/reduce';
import _default3 from 'ramda/src/curryN';
import _default2 from 'ramda/src/ifElse';
import _default from 'ramda/src/map';

import { isPlainObj, alwaysFunction, safeSpecTransforms } from './util';

var makeSpec = function makeSpec(transforms) {
  var spec = _default(_default2(isPlainObj, makeSpec, alwaysFunction), safeSpecTransforms(transforms));
  return _default3(_default4(_default5, 0, _default6('length', _default7(spec))), function () {
    for (var _len = arguments.length, allArgs = Array(_len), _key = 0; _key < _len; _key++) {
      allArgs[_key] = arguments[_key];
    }

    return _default(function (fn) {
      return _default8(fn, allArgs);
    }, spec);
  });
};

/**
 * A port over of [Ramda's applySpec()](http://ramdajs.com/docs/#applySpec),
 * but with a few changes you may or may not see as necessary.
 * (1) Properly curried so that you can optionally provide args as (spec, input) rather than only as (spec)(input)
 * (2) It wraps each function in a tryCatch that logs the key name of the transform function that fails.
 *
 * Using applySpec() in the wild and at-scale proved to be one of the hardest Ramda functions to debug.
 * Limiting its functionality and adding console logging doesn't sound like a necessary thing,
 * but when working with Ramda on a full team it may help prevent devs from getting so frustrated
 * that they vote to abandon the library while they're still learning the functional programming paradigm.
 *
 * @func
 * @sig {k: v} -> {k: v} -> {k: v}
 * @param {Object} spec An object whose values are functions which
 * expect the entire object to be fed in as input.
 * This is identical to the input you'd pass into applySpec()
 * @param {Object} input An object to be mapped over (transformed)
 * @returns {Object} The input object transformed to contain only the props named in the spec
 * (with the corresponding transforms applied at that key)
 */
export default _default9(function (transforms, input) {
  return makeSpec(transforms)(input);
});